@@ -1,11 +1,17 @@
 package Apache;
 use strict;
-use DynaLoader ();
-use Apache::Constants ();
+use Exporter ();
+use Apache::Constants qw(OK DECLINED);
 
-@Apache::ISA = qw(Exporter DynaLoader);
-@Apache::EXPORT_OK = qw(exit warn);
-$Apache::VERSION = "1.17";
+@Apache::EXPORT_OK = qw(exit warn fork forkoption);
+$Apache::VERSION = "1.20";
+
+*import = \&Exporter::import;
+
+unless(defined &bootstrap) {
+    require DynaLoader;
+    @Apache::ISA = qw(DynaLoader);
+}
 
 if (caller eq "CGI::Apache") {
     #we must die here outside of httpd so CGI::Switch works
@@ -15,8 +21,7 @@ else {
     eval { bootstrap Apache $Apache::VERSION; } 
 }
 if($@) {
-    my $gw = $ENV{GATEWAY_INTERFACE} || '';
-    die "$@\n" if substr($gw,0,8) eq 'CGI-Perl';
+    die "$@\n" if exists $ENV{MOD_PERL};
     warn "warning: can't `bootstrap Apache $Apache::VERSION' outside of httpd\n";
 }
 
@@ -29,7 +34,7 @@ sub parse_args {
     my($wantarray,$string) = @_;
     return unless defined $string and $string;
     if(defined $wantarray and $wantarray) {
-	return map { Apache::unescape_url_info($_) } split /[=&]/, $string;
+	return map { Apache::unescape_url_info($_) } split /[=&]/, $string, -1;
     }
     $string;
 }
@@ -61,7 +66,7 @@ sub read {
     $nrd = $total = 0;
     $buf = "";
     $_[1] ||= "";
-    #$#{$_[1]} = $bufsiz unless defined $_[1];
+    #$_[1] = " " x $bufsiz unless defined $_[1]; #XXX?
 
     $r->hard_timeout("Apache->read");
 
@@ -104,40 +109,29 @@ sub PRINTF {
 sub send_cgi_header {
     my($r, $headers) = @_;
     my $dlm = "\015?\012"; #a bit borrowed from LWP::UserAgent
-    my(@headerlines) = split /$dlm/, $headers;
     my($key, $val);
-
-    foreach (@headerlines) {
-	if (/^(\S+?):\s*(.*)$/) {
+    while(($_, $headers) = split /$dlm/, $headers, 2) {
+	#warn "hunk=`$_'\n";
+	#warn "rest=`$headers'\n";
+	if ($_ && /^(\S+?):\s*(.*)$/) {
 	    ($key, $val) = ($1, $2);
 	    last unless $key;
 	    $r->cgi_header_out($key, $val);
-	    if($key eq "Location") {
-		if($val =~ m,^/,) {
-		    #/* This redirect needs to be a GET no 
-                    #   matter what the original
-		    # * method was.
-
-		    $r->method("GET");
-		    $r->method_number(0); #M_GET 
-		    $r->internal_redirect_handler($val);
-		    return 0;
-		}
-	    }
 	}
 	else {
-	    #we didn't find the header terminator, punt away
-	    if(!$r->sent_header()) {
-		$r->send_http_header;
+	    #warn "mod_perl: found header terminator\n";
+	    my $not_sent = 0;
+	    if($Apache::__SendHeader) {
+		$not_sent = not $r->sent_header;
 	    }
-	    $r->print($_);
+	    else {
+		$not_sent = 1;
+	    }
+	    $r->send_http_header if $not_sent;
+	    $r->print($headers); #send rest of buffer, without stripping newlines!!!
+	    last;
 	}
     }
-    if($headers =~ /${dlm}${dlm}$/ or
-       $headers =~ /^(${dlm}){1,2}$/) 
-    {
-        $r->send_http_header;
-    }
 }
 
 sub as_string {
@@ -478,7 +472,7 @@ soon.
 
 If the configuration directive B<IdentityCheck> is set to on:  then the
 first time C<$r-E<gt>get_remote_logname> is called the server does an RFC
-1413 (ident) lookup to get the remote user's system name. Generally for
+1413 (ident) lookup to get the remote users system name. Generally for
 UNI* systems this is their login. The result is cached in C<$c-E<gt>remote_logname>
 then returned.  Subsequent calls to C<$r-E<gt>get_remote_host> return the
 cached value.
@@ -541,7 +535,7 @@ something else and implement your own authentication scheme.
 
 Returns a reference to the current value of the per directory
 configuration directive B<AuthName>.  The AuthName directive creates
-protection realm within the server's document space. To quote RFC 1945
+protection realm within the server document space. To quote RFC 1945
 "These realms allow the protected resources on a server to be
 partitioned into a set of protection spaces, each with its own
 authentication scheme and/or authorization database." The client uses
@@ -578,6 +572,11 @@ Return a reference to the server info object (blessed into the
 B<Apache::Server> package).  This is really a C<server_rec*> in
 disguise.  The following methods can be used on the server object:
 
+=item $s = Apache->server
+
+Same as above, but only available during server startup for use in
+<Perl> sections, B<PerlScript> or B<PerlModule>.
+
 =item $s->server_admin
 
 Returns the mail address of the person responsible for this server.
@@ -598,6 +597,14 @@ Returns true if this is a virtual server.
 
 Returns the wild-carded names for HostAlias servers. 
 
+=item $s->warn
+
+Alias for Apache::warn.
+
+=item $s->log_error
+
+Alias for Apache::log_error.
+
 =back
 
 =head1 SETTING UP THE RESPONSE
@@ -652,6 +659,26 @@ writing handlers in mod_perl you can use Perl variables for this.
    $r->notes("MY_HANDLER", OK);
    $val = $r->notes("MY_HANDLER");
 
+=item $r->subprocess_env( $key, [$value] )
+
+Return the value of a named entry in the Apache C<subprocess_env>
+table, or optionally set the value of a named entry. This table is
+used by mod_include.  By setting some custom variables inside
+a perl handler it is possible to combine perl with mod_include nicely.
+If you say, e.g. in a PerlHeaderParserHandler
+
+   $r->subprocess_env(MyLanguage => "de");
+
+you can then write in your .shtml document:
+
+   <!--#if expr="$MyLanguage = en" -->
+   English
+   <!--#elif expr="$MyLanguage = de" -->
+   Deutsch
+   <!--#else -->
+   Sorry
+   <!--#endif -->
+
 =item $r->content_type( [$newval] )
 
 Get or set the content type being sent to the client.  Content types
@@ -746,6 +773,39 @@ telling the client. For instance:
 
    $r->internal_redirect_handler("/home/sweet/home.html");
 
+=item $r->custom_response($code, $uri)
+
+This method provides a hook into the B<ErrorDocument> mechanism,
+allowing you to configure a custom response for a given response
+code at request-time.
+
+Example:
+
+    use Apache::Constants ':common';
+
+    sub handler {
+        my($r) = @_;
+
+        if($things_are_ok) {
+	    return OK;
+        }
+
+        #<Location $r->uri>
+        #ErrorDocument 401 /error.html
+        #</Location>
+
+        $r->custom_response(AUTH_REQUIRED, "/error.html");
+
+        #can send a string too
+        #<Location $r->uri>
+        #ErrorDocument 401 "sorry, go away"
+        #</Location>
+
+        #$r->custom_response(AUTH_REQUIRED, qq("sorry, go away"));
+
+        return AUTH_REQUIRED;
+    }
+
 =back
 
 =head1 SERVER CORE FUNCTIONS
@@ -874,7 +934,9 @@ Uh, oh.  Write a message to the server errorlog.
 
 =item $r->warn($message)
 
-An alias for Apache->log_error.
+For pre-1.3 versions of apache, this is just an alias for
+C<log_error>.  With 1.3+ versions of apache, this message will only be
+send to the error_log if B<LogLevel> is set to B<warn> or higher. 
 
 =back
 
