@@ -1,13 +1,13 @@
 package Apache::Registry;
 use Apache ();
 #use strict; #eval'd scripts will inherit hints
-use Apache::Constants qw(:common &OPT_EXECCGI);
+use Apache::Constants qw(:common &OPT_EXECCGI &REDIRECT);
 use FileHandle ();
 use File::Basename ();
 use Cwd ();
 
-#$Id: Registry.pm,v 1.36 1997/09/16 00:47:48 dougm Exp dougm $
-$Apache::Registry::VERSION = (qw$Revision: 1.36 $)[1];
+#$Id: Registry.pm,v 1.43 1997/12/02 04:39:46 dougm Exp $
+$Apache::Registry::VERSION = (qw$Revision: 1.43 $)[1];
 
 $Apache::Registry::Debug ||= 0;
 # 1 => log recompile in errorlog
@@ -18,14 +18,20 @@ Apache->module('Apache::Debug') if $Apache::Registry::Debug;
 my $Is_Win32 = $^O eq "MSWin32";
 
 sub handler {
-    my($r) = @_;
-    Apache->request($r);
+    my $r = shift;
+    if(ref $r) {
+	$r->request($r);
+    }
+    else {
+	#warn "Registry args are: ($r, @_)\n";
+	$r = Apache->request;
+    }
     my $filename = $r->filename;
     #local $0 = $filename; #this core dumps!?
     *0 = \$filename;
     my $oldwarn = $^W;
     $r->log_error("Apache::Registry::handler for $filename in process $$")
-	if $Debug & 4;
+	if $Debug && $Debug & 4;
 
     if (-r $filename && -s _) {
 	if (!($r->allow_options & OPT_EXECCGI)) {
@@ -37,7 +43,7 @@ sub handler {
 	    $r->log_reason("attempt to invoke directory as script", $filename);
 	    return FORBIDDEN;
 	}
-	unless ($Is_Win32 or -x _) {
+	unless (-x _ or $Is_Win32) {
 	    $r->log_reason("file permissions deny server execution",
 			   $filename);
 	    return FORBIDDEN;
@@ -47,14 +53,16 @@ sub handler {
 
 	# turn into a package name
 	$r->log_error(sprintf "Apache::Registry::handler examining %s",
-		      $r->uri) if $Debug & 4;
+		      $r->uri) if $Debug && $Debug & 4;
 	my $script_name = $r->path_info ?
 	    substr($r->uri, 0, length($r->uri)-length($r->path_info)) :
 		$r->uri;
 
-	my $srv = $r->server;
-	$script_name = join "", $srv->server_hostname, $script_name
-	    if $srv->is_virtual;
+	if($Apache::Registry::NameWithVirtualHost) {
+	    my $srv = $r->server;
+	    $script_name = join "", $srv->server_hostname, $script_name
+		if $srv->is_virtual;
+	}
 
 	# Escape everything into valid perl identifiers
 	$script_name =~ s/([^A-Za-z0-9\/])/sprintf("_%2x",unpack("C",$1))/eg;
@@ -70,33 +78,32 @@ sub handler {
 	my $package = "Apache::ROOT$script_name";
 	$Apache::Registry::curstash = $script_name;
 	$r->log_error("Apache::Registry::handler package $package")
-	   if $Debug & 4;
+	   if $Debug && $Debug & 4;
 
 
 	my $cwd = Cwd::fastcwd();
 	chdir File::Basename::dirname($r->filename);
 
 	if (
-	    defined($Apache::Registry->{$package}{mtime})
+	    exists $Apache::Registry->{$package}{mtime}
 	    &&
 	    $Apache::Registry->{$package}{mtime} <= $mtime
 	   ){
 	    # we have compiled this subroutine already, nothing left to do
  	} else {
 	    $r->log_error("Apache::Registry::handler reading $filename")
-		if $Debug & 4;
+		if $Debug && $Debug & 4;
 	    my($sub);
 	    {
 		my $fh = FileHandle->new($filename);
-		(my $oldrs, local $/) = ($/, undef);
+		local $/;
 		$sub = <$fh>;
 		$sub = parse_cmdline($sub);
-		$/ = $oldrs;
 	    }
 
 	    # compile this subroutine into the uniq package name
-            $r->log_error("Apache::Registry::handler eval-ing") if $Debug & 4;
- 	    undef &{"$package\::handler"} unless $Debug & 4; #avoid warnings
+            $r->log_error("Apache::Registry::handler eval-ing") if $Debug && $Debug & 4;
+ 	    undef &{"$package\::handler"} unless $Debug && $Debug & 4; #avoid warnings
 	    $r->clear_rgy_endav($script_name);
 
 	    my $eval = join(
@@ -112,15 +119,16 @@ sub handler {
 	    compile($eval);
 	    if ($@) {
 		$r->log_error($@);
-		return SERVER_ERROR unless $Debug & 2;
+		return SERVER_ERROR unless $Debug && $Debug & 2;
 		return Apache::Debug::dump($r, SERVER_ERROR);
 	    }
             $r->log_error(qq{Compiled package \"$package\" for process $$})
-	       if $Debug & 1;
+	       if $Debug && $Debug & 1;
 	    $Apache::Registry->{$package}{mtime} = $mtime;
 	}
 
-	eval {$package->handler;};
+	my $cv = \&{"$package\::handler"};
+	eval { &{$cv}($r, @_) } if $r->seqno;
 	{
 	    local $^W = 0; #shutup Cwd.pm
 	    chdir $cwd;
@@ -128,20 +136,27 @@ sub handler {
 	$^W = $oldwarn;
 
 	my $err = $@;
-	if($INC{'CGI.pm'}) {
-	    use Exporter ();
+	if(exists $INC{'CGI.pm'}
+	   && ! $Apache::Registry::CGI_versioncheck_done ) {
+	    $Apache::Registry::CGI_versioncheck_done++;
+	    require Exporter;
 	    eval { Exporter::require_version('CGI', 2.36); }; 
 	    $err .= $@ if $@;
 	}
 	if ($err) {
 	    $r->log_error($err);
-	    return SERVER_ERROR unless $Debug & 2;
+	    return SERVER_ERROR unless $Debug && $Debug & 2;
 	    return Apache::Debug::dump($r, SERVER_ERROR);
 	}
 
+	if(my $loc = $r->header_out("Location")) {
+	    if($r->status == 200 and substr($loc, 0, 1) ne "/") {
+		return REDIRECT;
+	    }
+	}
 	return $r->status;
     } else {
-	return NOT_FOUND unless $Debug & 2;
+	return NOT_FOUND unless $Debug && $Debug & 2;
 	return Apache::Debug::dump($r, NOT_FOUND);
     }
 }
@@ -186,6 +201,8 @@ sub parse_cmdline {
 #trick so we show up under CPAN/modules/by-module/CGI/
 package CGI::mod_perl;
 
+sub DESTROY {}
+
 1;
 
 __END__
