@@ -1,10 +1,10 @@
 package Apache::Status;
 use strict;
-use vars qw($VERSION);
 
-$VERSION = (qw$Revision: 1.13 $)[1];
+$Apache::Status::VERSION = (qw$Revision: 1.18 $)[1];
 
 my(%status) = (
+   script => "Loaded PerlScripts",
    inc => "Loaded Modules",
    rgysubs => "Compiled Registry Scripts",
    symdump => "Symbol Table Dump",
@@ -28,16 +28,16 @@ sub handler {
     no strict 'refs';
     header($r);
     if(defined &$sub) {
-	$r->module('CGI::Switch');
+	$r->module('CGI');
 	$r->module('Devel::Symdump');
         eval { Exporter::require_version('Devel::Symdump', 2.00); };
 	if($@) {
 	    $r->print("You need to install Devel::Symdump version 2.00 or higher!<p>\n$@");
 	    return 1;
 	}		      
-	$r->print(@{ &{$sub}($r,CGI::Switch->new) });
+	$r->print(@{ &{$sub}($r,CGI->new) });
     }
-    elsif (exists $Apache::Registry->{$qs}) { 
+    elsif ($qs and defined %{$qs."::"}) {
 	$r->module('Devel::Symdump');
 	$r->print(symdump($qs));
     }
@@ -78,22 +78,36 @@ sub status_symdump { [symdump('main')] }
 
 sub status_inc {
     my($r,$q) = @_;
-    my(@retval, $module, $v);
-    foreach $module ($q->param("INC")) {
-	delete $INC{$module};
-    }
-    $q->delete("INC");
-    foreach $module (sort keys %INC) {
-	next unless $INC{$module}; #e.g. fake Apache/TieHandle.pm
+    my(@retval, $module, $v, $file);
+    my $uri = $r->uri;
+    push @retval, "<table border=1>";
+    push @retval, "<tr><td><b>Package</b></td><td><b>Version</b><td><b>File</b></td></tr>";
+    foreach $file (sort keys %INC) {
+	next if $file =~ m:^/:;
+	next unless $file =~ m:\.pm:;
+	next unless $INC{$file}; #e.g. fake Apache/TieHandle.pm
 	no strict 'refs';
-	$module =~ s,/,::,g;
+	($module = $file) =~ s,/,::,g;
 	$module =~ s,\.pm$,,;
 	$v = ${"$module\:\:VERSION"} || '0.00';
 	push @retval, 
-	"$module ($v) <br>\n";
-	#$q->checkbox(-name => "INC", -value => $_, -label => $_), "<br>";
+	qq(<tr><td><a href="$uri?$module">$module</a></td><td>$v</td><td>$INC{$file}</td></tr>);
     }
-    push @retval, "<hr>"; #, $q->submit(-value => "Delete");
+    push @retval, "</table>";
+    \@retval;
+}
+
+sub status_script {
+    my($r,$q) = @_;
+    my(@retval, $file);
+    push @retval, "<table border=1>";
+    push @retval, "<tr><td><b>PerlScript</b></td><td><b>File</b></td></tr>";
+    foreach $file (sort keys %INC) {
+	next if $file =~ m:\.pm$:;
+	push @retval, 
+	qq(<tr><td>$file</td><td>$INC{$file}</td></tr>);
+    }
+    push @retval, "</table>";
     \@retval;
 }
 
@@ -102,7 +116,7 @@ sub status_rgysubs {
     my(@retval);
     local $_;
     my $uri = $r->uri;
-    push @retval, "<b>Click on package name to see it's symbol table</b><p>\n";
+    push @retval, "<b>Click on package name to see its symbol table</b><p>\n";
     foreach (sort keys %{$Apache::Registry}) {
 	push @retval, 
 	#$q->checkbox(-name => "RGY", 
