@@ -1,3 +1,8 @@
+AddType text/x-server-parsed-html .shtml
+AddType text/perl-module .pm
+
+Action text/perl-module /perl/action.pl
+
 #these three are passed to perl_parse(), 
 #which happens before <Perl> sections are processed
 #optionally, they can be inside <Perl>, however for testing we want
@@ -5,6 +10,8 @@
 #besides that, we rely on the PerlScript below to set @INC to our blib
 
 PerlScript docs/startup.pl
+PerlScript docs/stacked.pl
+
 #-Tw
 PerlTaintCheck On
 PerlWarn On
@@ -15,10 +22,12 @@ PerlSetVar KeyForPerlSetVar OK
 #!perl
 #line 16 httpd.conf
 
+use Apache::Constants qw(MODULE_MAGIC_NUMBER);
 use IO::Handle ();
 use Cwd qw(fastcwd);
 my $dir = join "/", fastcwd, "t";
 my $Is_Win32 = ($^O eq "MSWin32");
+my $mmn = MODULE_MAGIC_NUMBER;
 
 sub prompt ($;$) {
     my($mess,$def) = @_;
@@ -45,11 +54,11 @@ if($User eq "root") {
 print "Will run tests as User: '$User' Group: '$Group'\n";
 
 $Port = 8529;
-$ServerName = "localhost";
 $DocumentRoot = "$dir/docs";
+$ServerName = "localhost";
 
 push @AddType, ["text/x-server-parsed-html" => ".shtml"];
-
+ 
 for (qw(/perl/ /cgi-bin/)) {
     push @Alias, [$_ => "$dir/net/perl/"];
 }
@@ -71,12 +80,22 @@ $Location{"/cgi-bin"} = {
     Options    => "ExecCGI",
 };
 
-$Location{"/perl/io"} = {
-    @mod_perl,
-    PerlSendHeader => "On",
-    PerlSetupEnv   => "On",
+$VirtualHost{"localhost"} = {
+    Location => {
+	"/perl/io" => {
+	    @mod_perl,
+	    PerlSendHeader => "On",
+	    PerlSetupEnv   => "On",
+	},
+    },
 };
 
+#$Location{"/perl/io"} = {
+#    @mod_perl,
+#    PerlSendHeader => "On",
+#    PerlSetupEnv   => "On",
+#};
+
 $Location{"/perl/perl-status"} = {
     SetHandler  => "perl-script",
     PerlHandler => "Apache::Status",
@@ -99,4 +118,18 @@ $LockFile = "/tmp/mod_perl.lock";
 #push @PerlChildInitHandler, "My::child_init";
 #push @PerlChildExitHandler, "My::child_exit";
 
+$Location{"/STAGE"} = {
+    ErrorDocument => [
+	      [403 => "/stage-redir"],
+	      [404 => "/stage-redir"],
+    ],
+};
+
+$Location{"/stage-redir"} = {
+    @mod_perl,
+    PerlHandler => "Apache::Stage",
+};
+
+$PerlTransHandler =  "PerlTransHandler::handler";
+
 </Perl>
